{{ Stpr_PWM__ASM.spin    BLDC_PWM_ASM.spin DOKU:  CMap0081, CMap0237, CMap0217 }}
{{
2011_10_04na: Initial version, start to rebuild driver from BLDC-PWM

This module creates the control signals for two channels of H-bridge PWMed power stages.
Can be used for two phase bipolar stepper motors.
Also possible: unipolar stepper motors with freewheeling diodes
Also possible: drive two DC-motors in 4 quadrants

Output Pins:  9: test,  8: sync, 7: /Bt 6: /Bb 5: Bt  4: Bb  3: /At  2: /Ab  1: At  0: Ab
/Bt means: complement output of channel B top Fet. And so on
}}
CON
  MaxCmd   = 3   ' two different commands allowed, number must be one more
  #1, CmdIdle, CmdStar

VAR
  byte ID_cog                    ' Global Variable number of the Cog, value is returned to calling process.

PUB Start( GlobPtr, AddrOffs ) | i   ' Start process in Cog: Pointer to global PCB (Process Control Block) and an offset

  Stop  ' Start can restart, therefore stop a possibly running cog first
        ' Create a local copy of the global pointer array and do the address correction 
  repeat i from 0 to LONG[LONG[GlobPtr +   0]+AddrOffs]-1        ' The first parameter is the number of parameters
    LONG[@LoclPtr + (i<<2)] := LONG[GlobPtr + (i<<2)] + AddrOffs ' 32 bit pointers, but pointer address is byte oriented

  ifnot ID_cog
    ID_cog := cognew(@ASM_Entry, 0) + 1
  Result := ID_Cog

PRI Stop      ' stops a running cog
  if ID_cog
    cogstop(ID_cog~ - 1)

DAT     org   0   ' Assembly code for cog is located in the Data segment, org 0 needed for proper address generation

ASM_Entry       ' Process to generate the PWM-Signal: setting up of the times and patterns
        mov      Command,   #0                    ' Command = 0
        wrlong   Command,   CommandPtr            ' Mirror command globally: shows to public: Process running

        mov      outa,      #0                    ' reset all pins low
        mov      dira,      dirA_Sel              ' sets direction of portA pins  ( in this case: 10 Output pins)

:LoopIdl
        rdlong    Command,  CommandPtr            ' Test for new command
        mov       outa,     #0                    ' Reset debug test pin (working only if output is activated)
        xor       outa,     SynMskO
        cmp       Command,  #MaxCmd       wc      ' compare new Command to maximum number
if_nc   mov       Command,  #0                    ' ignore if out or range! do nothing
        add       Command,  #:JmpTab              ' add address of Jmptable to get entry
        jmp       Command                         ' jump to the table
                                                  
:JmpTab
        jmp      #:LoopIdl                        ' 0:  Idle, wait for command
        jmp      #:Go_Idle                        ' 1:  At Idle Mode
        jmp      #:StrtPWM                        ' 2:  PWM signals running

'---------------- specific Code starts here -----------

:Go_Idle
        mov      outa,      #0                    ' reset all pins low 
        mov      Command,   #0                    ' 
        wrlong   Command,   CommandPtr            ' Command acknowledge: done, now Idle
                                                     
        jmp      #:LoopIdl                        ' wait for new command


:StrtPWM    ' PWM running, command and then confirm loops to Idle-Command
        mov      Command,   #0                    ' Command <- 0
        wrlong   Command,   CommandPtr            ' Function acknowledge (Command -> *CommandPtr)
        rdlong   NxtPatt,   PWMPattPtr            ' read actual pattern group
        shl      NxtPatt,   #3                    ' times 8 
        add      NxtPatt,   #PattGrp              ' pointer to table
        
        mov      PWMSTim,   cnt                   ' Set start time of actual Cycle
        rdlong   NxtTra1,   PWMTra1Ptr            ' Time of first transition
        rdlong   NxtTra2,   PWMTra2Ptr            ' Time of second transition


        ' enter the loop

:LoopRun

        rdlong   Command,   CommandPtr            '7-22 Check for Idle-Command   
        cmp      Command,   #CmdIdle      wz      '4    

if_z    jmp      #:Go_Idle                         '4    Stop PWMing, wait for new command

        mov      OutPatt,   NxtPatt               ' Pattern pointer saved over this cycle
        movs     :S1,       OutPatt   ' Modify code: implat source address
        nop                             '
:S1     mov      outa,      0_0                   ' Start pattern of this cycle      

'      wrlong   NxtPatt,   NxtPattPtr            ' read actual pattern group
'      wrlong   OutPatt,   OutPattPtr            ' read actual pattern group
 
        add      OutPatt,   #1                    ' increment to next pattern
        or       outa,      SynMskO               '   *-v- set the sync bit
                                                  ' now the Pins are active and stay for at least ~ 40% of cycle

        call     #UpdatePara                       ' Update: Patterngroup, Mode, transition times

        and       outa,     SynMskA               '   *-^- reset the sync bit: new setvalues allowed
:loopTr1                                          ' loop to reach first transition
          xor      outa,      DebMskO
        mov      ScrReg1,   NxtTra1
        sub      ScrReg1,   cnt
        cmps      ScrReg1,   #0              wc    ' test for negative
'      wrlong   ScrReg1,   OutPattPtr            ' read actual pattern group
          xor      outa,      DebMskO
if_nc   jmp      #:loopTr1                        ' loop until transition time elapses
          xor      outa,      DebMskO

        movs     :S2,       OutPatt   ' Modify code: implat source address
        nop                             '
:S2        mov      outa,      0_0               ' Deadtime
        add      OutPatt,   #1                    ' increment to next pattern
        NOP
        NOP
        NOP
        movs     :S3,       OutPatt   ' Modify code: implat source address
        nop                             '
:S3        mov      outa,      0_0               ' Deadtime
        add      OutPatt,   #1                    ' increment to next pattern

:loopTr2                                          ' loop to reach second transition
          xor      outa,      DebMskO
        mov      ScrReg1,   NxtTra2
        sub      ScrReg1,   cnt
        cmps      ScrReg1,   #0              wc    ' test for negative
          xor      outa,      DebMskO
if_nc   jmp      #:loopTr2                        ' loop until transition time elapses
          xor      outa,      DebMskO

        movs     :S4,       OutPatt   ' Modify code: implat source address
        nop                             '
:S4        mov      outa,      0_0               ' Deadtime
        add      OutPatt,   #1                    ' increment to next pattern
        NOP
        NOP
        NOP
        movs     :S5,       OutPatt   ' Modify code: implat source address
        nop                             '
:S5        mov      outa,      0_0               ' Deadtime
        add      OutPatt,   #1                    ' increment to next pattern

:loopCyc                                          ' loop to reach end of cycle
        mov      ScrReg1,   PWMSTim
        sub      ScrReg1,   cnt
        cmps      ScrReg1,   #0              wc    ' test for negative
if_nc   jmp      #:loopCyc                        ' loop until cycle time elapses

        movs     :S6,       OutPatt   ' Modify code: implat source address
        nop                             '
:S6        mov      outa,      0_0               ' Deadtime
        NOP
        NOP
        NOP        

        jmp      #:LoopRun                        '4   go to :LoopRun



'----------------------------------------------
UpdatePara
        rdlong   PWMPrdy,   PWMPrdyPtr            ' Check for new parameter set
        add      PWMPrdy,   #0            wz      ' test for not zero
if_nz   rdlong   PWMCycl,   PWMCyclPtr            ' Cycle Time in Clocks  (PWMCycl <- *PWMCyclPtr)
if_nz   rdlong   PWMMode,   PWMModePtr            ' read actual pattern group
if_nz   rdlong   NxtPatt,   PWMPattPtr            ' read actual pattern group
if_nz   shl      NxtPatt,   #3                    ' times 8 
if_nz   add      NxtPatt,   #PattGrp              ' pointer to table : valid in next cycle
if_nz   rdlong   PWMTra1,   PWMTra1Ptr            ' read actual pattern group
if_nz   rdlong   PWMTra2,   PWMTra2Ptr            ' read actual pattern group

        mov      NxtTra1,   PWMTra1               ' Reload NxtTransition
        mov      NxtTra2,   PWMTra2               ' Reload NxtTransition
        add      NxtTra1,   PWMSTim               ' +offset start time
        add      NxtTra2,   PWMSTim               ' +offset start time
        add      PWMSTim,   PWMCycl               ' Next PWM cycle start time (end of this cycle)
if_nz   mov      PWMPrdy,   #0                    ' Reset the flag
if_nz   wrlong   PWMPrdy,   PWMPrdyPtr            ' and send acknowledge

UpdatePara_ret ret           ' return from subroutine
'----------------------------------------------

LoclPtr  ' All Pointers to the global memory
CommandPtr  long  0   ' PWM-Generator
PWMPrdyPtr  long  0   ' Flag: new parameter set ready
PWMCyclPtr  long  0   ' Cycle time
PWMPattPtr  long  0   ' Number of PWM Pattern Group
PWMModePtr  long  0   ' Mode: 0: normal mode, 1: only 1 transition, 2: no transition at all 
PWMTra1Ptr  long  0   ' The first transition
PWMTra2Ptr  long  0   ' The second transition

OutPattPtr     long  0   ' Pointer to the actual pattern to output
NxtPattPtr     long  0   ' Pointer to pattern of next cycle
NxtTra1Ptr     long  0   ' The first transition of this cycle                                              
NxtTra2Ptr     long  0   ' The second transition of this cycle
PWMSTimPtr     long  0   ' start of the next cycle


' Local variables: mirror for interprocess communication
Command     long  0   ' Command to process
PWMPrdy     long  0   ' Flag: new parameter set ready
PWMCycl     long  0   ' Cycle time                                                         
PWMPatt     long  0   ' Number of PWM Pattern Group                                       
PWMMode     long  0   ' Mode: 0: normal mode, 1: only 1 transition, 2: no transition at all
PWMTra1     long  0   ' The first transition                                               
PWMTra2     long  0   ' The second transition                                              

' other Local variables
OutPatt     long  0   ' Pointer to the actual pattern to output
NxtPatt     long  0   ' Pointer to pattern of next cycle
NxtTra1     long  0   ' The first transition of this cycle                                              
NxtTra2     long  0   ' The second transition of this cycle
PWMSTim     long  0   ' start of the next cycle


DirA_Sel    long  %11_1111_1111     ' These bits are "OUT"                                       
SynMskA     long  %10_1111_1111     ' mask to reset Sync Bit by "AND"                                        
SynMskO     long  %01_0000_0000     ' mask to set Sync Bit by "OR"
DebMskA     long  %01_1111_1111     ' mask to reset Debugging Bit by "AND"                                        
DebMskO     long  %10_0000_0000     ' mask to set   Debugging Bit by "OR"

ScrReg1     long  0   ' scratch register                              

PattGrp 'case 0:  A and B channel positive, A > B
        '            Bt Bb At Ab
            long   %10_01_10_01     ' Trans 0 :  B and A pulse positive
            long   %00_00_10_01     '       1 :  Dead time for B               
            long   %01_10_10_01     '       2 :  B started pause
            long   %01_10_00_00     '       3 :  Dead time for A               
            long   %01_10_01_10     '       4 :  A started pause
            long   %00_00_00_00     '       5 :  Dead time Cycle end
            long   0, 0

        'case 1: A and B channel positive, B > A
        '            Bt Bb At Ab
            long   %10_01_10_01     ' Trans 0 :  B and A pulse positive
            long   %10_01_00_00     '       1 :  Dead time for A               
            long   %10_01_01_10     '       2 :  A started pause
            long   %00_00_01_10     '       3 :  Dead time for B               
            long   %01_10_01_10     '       4 :  B started pause
            long   %00_00_00_00     '       5 :  Dead time Cycle end
            long   0, 0


        'case 2:  A negative, B channel positive, B > A
        '            Bt Bb At Ab
            long   %10_01_01_10     ' Trans 0 :  B pulse A pause
            long   %10_01_00_00     '       1 :  Dead time for A               
            long   %10_01_10_01     '       2 :  A started pulse
            long   %00_00_10_01     '       3 :  Dead time for B               
            long   %01_10_10_01     '       4 :  B started pause
            long   %00_00_00_00     '       5 :  Dead time Cycle end
            long   0, 0

        'case 3:  A negative, B channel positive, A > B
        '            Bt Bb At Ab
            long   %10_01_01_10     ' Trans 0 :  B pulse A pause
            long   %00_00_01_10     '       1 :  Dead time for B               
            long   %01_10_01_10     '       2 :  B started pause
            long   %01_10_00_00     '       3 :  Dead time for A               
            long   %01_10_10_01     '       4 :  A started pulse
            long   %00_00_00_00     '       5 :  Dead time Cycle end
            long   0, 0

        'case 4:  A negative, B channel negative, A > B
        '            Bt Bb At Ab
            long   %01_10_01_10     ' Trans 0 :  B pause A pause
            long   %00_00_01_10     '       1 :  Dead time for B               
            long   %10_01_01_10     '       2 :  B started pulse
            long   %10_01_00_00     '       3 :  Dead time for A               
            long   %10_01_10_01     '       4 :  A started pulse
            long   %00_00_00_00     '       5 :  Dead time Cycle end
            long   0, 0

        'case 5:  A negative, B channel negative, B > A
        '            Bt Bb At Ab
            long   %01_10_01_10     ' Trans 0 :  B pause A pause
            long   %01_10_00_00     '       1 :  Dead time for A               
            long   %01_10_10_01     '       2 :  A started pulse
            long   %00_00_10_01     '       3 :  Dead time for B               
            long   %10_01_10_01     '       4 :  B started pulse
            long   %00_00_00_00     '       5 :  Dead time Cycle end
            long   0, 0

        'case 6:  A positive, B channel negative, B > A
        '            Bt Bb At Ab
            long   %01_10_10_01     ' Trans 0 :  B pause A pulse
            long   %01_10_00_00     '       1 :  Dead time for A               
            long   %01_10_01_10     '       2 :  A started pause
            long   %00_00_01_10     '       3 :  Dead time for B               
            long   %10_01_01_10     '       4 :  B started pulse
            long   %00_00_00_00     '       5 :  Dead time Cycle end
            long   0, 0

        'case 7:  A positive, B channel negative, A > B
        '            Bt Bb At Ab
            long   %01_10_10_01     ' Trans 0 :  B pause A pulse
            long   %00_00_10_01     '       1 :  Dead time for B               
            long   %10_01_10_01     '       2 :  B started pulse
            long   %10_01_00_00     '       3 :  Dead time for A               
            long   %10_01_01_10     '       4 :  A started pause
            long   %00_00_00_00     '       5 :  Dead time Cycle end
            long   0, 0

FIT